@@ -29,6 +29,7 @@
 #include <sys/types.h>
 #include <stdio.h>
 #include <fcntl.h>
+#include <errno.h>
 
 #include <glib/gstdio.h>
 
@@ -57,6 +58,9 @@
 
 #define IN_USE_FILENAME               ".meta.isrunning"
 
+/* Stamp filename to check for first index */
+#define FIRST_INDEX_STAMP_FILENAME    ".firstindex"
+
 typedef enum {
 	TRACKER_DB_LOCATION_DATA_DIR,
 	TRACKER_DB_LOCATION_USER_DATA_DIR,
@@ -267,6 +271,12 @@ db_set_params (TrackerDBInterface *iface,
 	tracker_db_interface_execute_query (iface, NULL, "PRAGMA encoding = \"UTF-8\"");
 	tracker_db_interface_execute_query (iface, NULL, "PRAGMA auto_vacuum = 0;");
 
+	/* if we add direct access library (or enable running queries in parallel with updates,
+	   we need to disable read_uncommitted again
+	   however, when read_uncommitted is disabled, we need the custom page cache (wrapper)
+	   to avoid locking issues between long running batch transactions and queries */
+	tracker_db_interface_execute_query (iface, NULL, "PRAGMA read_uncommitted = True;");
+
 	if (page_size != TRACKER_DB_PAGE_SIZE_DONT_SET) {
 		g_message ("  Setting page size to %d", page_size);
 		tracker_db_interface_execute_query (iface, NULL, "PRAGMA page_size = %d", page_size);
@@ -406,6 +416,9 @@ db_manager_remove_all (gboolean rm_journal)
 
 	g_message ("Removing all database files");
 
+	/* Remove stamp file */
+	tracker_db_manager_set_first_index_done (FALSE);
+
 	/* NOTE: We don't have to be initialized for this so we
 	 * calculate the absolute directories here.
 	 */
@@ -549,7 +562,12 @@ tracker_db_manager_ensure_locale (void)
 
 	common = dbs[TRACKER_DB_METADATA].iface;
 
-	stmt = tracker_db_interface_create_statement (common, "SELECT OptionValue FROM Options WHERE OptionKey = 'CollationLocale'");
+	stmt = tracker_db_interface_create_statement (common, NULL, "SELECT OptionValue FROM Options WHERE OptionKey = 'CollationLocale'");
+
+	if (!stmt) {
+		return;
+	}
+
 	result_set = tracker_db_statement_execute (stmt, NULL);
 	g_object_unref (stmt);
 
@@ -562,10 +580,13 @@ tracker_db_manager_ensure_locale (void)
 		/* Locales differ, update collate keys */
 		g_message ("Updating DB locale dependent data to: %s\n", current_locale);
 
-		stmt = tracker_db_interface_create_statement (common, "UPDATE Options SET OptionValue = ? WHERE OptionKey = 'CollationLocale'");
-		tracker_db_statement_bind_text (stmt, 0, current_locale);
-		tracker_db_statement_execute (stmt, NULL);
-		g_object_unref (stmt);
+		stmt = tracker_db_interface_create_statement (common, NULL, "UPDATE Options SET OptionValue = ? WHERE OptionKey = 'CollationLocale'");
+
+		if (stmt) {
+			tracker_db_statement_bind_text (stmt, 0, current_locale);
+			tracker_db_statement_execute (stmt, NULL);
+			g_object_unref (stmt);
+		}
 	}
 
 	g_free (stored_locale);
@@ -633,9 +654,20 @@ static void
 free_thread_interface (gpointer data)
 {
 	TrackerDBInterface *interface = data;
+	GHashTableIter iter;
+	gpointer value;
 
 	g_static_mutex_lock (&thread_ifaces_mutex);
-	g_hash_table_remove (thread_ifaces, g_thread_self ());
+
+	g_hash_table_iter_init (&iter, thread_ifaces);
+
+	while (g_hash_table_iter_next (&iter, NULL, &value)) {
+		if (value == data) {
+			g_hash_table_iter_remove (&iter);
+			break;
+		}
+	}
+
 	g_static_mutex_unlock (&thread_ifaces_mutex);
 
 	g_object_unref (interface);
@@ -799,6 +831,9 @@ tracker_db_manager_init (TrackerDBManagerFlags  flags,
 			return FALSE;
 		}
 
+		/* Clear the first-index stamp file */
+		tracker_db_manager_set_first_index_done (FALSE);
+
 		db_recreate_all ();
 
 		/* Load databases */
@@ -818,7 +853,7 @@ tracker_db_manager_init (TrackerDBManagerFlags  flags,
 			g_message ("Didn't shut down cleanly last time, doing integrity checks");
 
 			for (i = 1; i < G_N_ELEMENTS (dbs) && !must_recreate; i++) {
-				TrackerDBCursor *cursor;
+				TrackerDBCursor *cursor = NULL;
 				TrackerDBStatement *stmt;
 				struct stat st;
 
@@ -839,14 +874,16 @@ tracker_db_manager_init (TrackerDBManagerFlags  flags,
 
 				loaded = TRUE;
 
-				stmt = tracker_db_interface_create_statement (dbs[i].iface,
+				stmt = tracker_db_interface_create_statement (dbs[i].iface, NULL,
 				                                              "PRAGMA integrity_check(1)");
 
-				cursor = tracker_db_statement_start_cursor (stmt, NULL);
-				g_object_unref (stmt);
+				if (stmt) {
+					cursor = tracker_db_statement_start_cursor (stmt, NULL);
+					g_object_unref (stmt);
+				}
 
 				if (cursor) {
-					if (tracker_db_cursor_iter_next (cursor)) {
+					if (tracker_db_cursor_iter_next (cursor, NULL)) {
 						if (g_strcmp0 (tracker_db_cursor_get_string (cursor, 0), "ok") != 0) {
 							must_recreate = TRUE;
 						}
@@ -889,8 +926,12 @@ tracker_db_manager_init (TrackerDBManagerFlags  flags,
 	in_use_file = g_open (in_use_filename,
 	                      O_WRONLY | O_APPEND | O_CREAT | O_SYNC,
 	                      S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP);
-	fsync (in_use_file);
-	close (in_use_file);
+
+        if (in_use_file >= 0) {
+                fsync (in_use_file);
+                close (in_use_file);
+        }
+
 	g_free (in_use_filename);
 
 	tracker_db_manager_ensure_locale ();
@@ -1247,6 +1288,83 @@ tracker_db_manager_has_enough_space  (void)
 	return tracker_file_system_has_enough_space (data_dir, TRACKER_DB_MIN_REQUIRED_SPACE, FALSE);
 }
 
+
+static gchar *
+get_first_index_stamp_path (void)
+{
+	return g_build_filename (g_get_user_cache_dir (),
+	                         "tracker",
+	                         FIRST_INDEX_STAMP_FILENAME,
+	                         NULL);
+}
+
+/**
+ * tracker_db_manager_get_first_index_done:
+ *
+ * Check if first full index of files was already done.
+ *
+ * Returns: %TRUE if a first full index have been done, %FALSE otherwise.
+ **/
+gboolean
+tracker_db_manager_get_first_index_done (void)
+{
+	gboolean exists;
+	gchar *stamp;
+
+	stamp = get_first_index_stamp_path();
+	exists = g_file_test (stamp, G_FILE_TEST_EXISTS);
+	g_free (stamp);
+
+	return exists;
+}
+
+/**
+ * tracker_db_manager_set_first_index_done:
+ *
+ * Set the status of the first full index of files. Should be set to
+ *  %FALSE if the index was never done or if a reindex is needed. When
+ *  the index is completed, should be set to %TRUE.
+ **/
+void
+tracker_db_manager_set_first_index_done (gboolean done)
+{
+	gboolean already_exists;
+	gchar *stamp;
+
+	stamp = get_first_index_stamp_path ();
+
+	already_exists = g_file_test (stamp, G_FILE_TEST_EXISTS);
+
+	if (done && !already_exists) {
+		GError *error = NULL;
+
+		/* If done, create stamp file if not already there */
+		if (!g_file_set_contents (stamp, "", -1, &error)) {
+			g_warning ("  Creating first-index stamp in "
+			           "'%s' failed: '%s'",
+			           stamp,
+			           error->message);
+			g_error_free (error);
+		} else {
+			g_message ("  First-index stamp created in '%s'",
+			           stamp);
+		}
+	} else if (!done && already_exists) {
+		/* If NOT done, remove stamp file */
+		if (g_remove (stamp)) {
+			g_warning ("  Removing first-index stamp from '%s' "
+			           "failed: '%s'",
+			           stamp,
+			           g_strerror (errno));
+		} else {
+			g_message ("  First-index stamp removed from '%s'",
+			           stamp);
+		}
+	}
+
+	g_free (stamp);
+}
+
 /**
  * tracker_db_manager_interrupt_thread:
  * @thread: a #GThread to be interrupted
@@ -1270,3 +1388,26 @@ tracker_db_manager_interrupt_thread (GThread *thread)
 
 	return tracker_db_interface_interrupt (interface);
 }
+
+/**
+ * tracker_db_manager_interrupt_thread_reset:
+ * @thread: a #GThread to be reset
+ *
+ * Reset @thread's interrupt state
+ *
+ **/
+void
+tracker_db_manager_interrupt_thread_reset (GThread *thread)
+{
+	TrackerDBInterface *interface;
+
+	g_static_mutex_lock (&thread_ifaces_mutex);
+	interface = g_hash_table_lookup (thread_ifaces, thread);
+	g_static_mutex_unlock (&thread_ifaces_mutex);
+
+	if (!interface) {
+		return;
+	}
+
+	tracker_db_interface_reset_interrupt (interface);
+}
