@@ -446,8 +446,10 @@ pool_dispatch_cb (gpointer data,
 
 	if (task->type == TRACKER_STORE_TASK_TYPE_QUERY) {
 		TrackerDBCursor *cursor;
+		GThread *running_thread = g_thread_self ();
 
-		task->data.query.running_thread = g_thread_self ();
+		tracker_data_manager_interrupt_thread_reset (running_thread);
+		task->data.query.running_thread = running_thread;
 		cursor = tracker_data_query_sparql_cursor (task->data.query.query, &task->error);
 
 		task->data.query.thread_data = task->callback.query.in_thread (cursor, task->error, task->user_data);
